###

Stuart Cullen - November 2014

To turn on coffee script compilation run "sh activate_coffee.sh" located
at the root of the project.  This process will continue to watch for any
changes to *.coffee files in the "Resource directory".

The resultant compiled "js" files have the same path and name as the 
coffee script and always overwrite the previous js files. 

###

###*
	Base class for trip plan option groups, it should keep a running
	total for the select atleast one, but it wasn't working so I've
	done a for loop for now...

	@todo there shouldn't be globals accessed in here or indeed anywhere
	@todo the actual row method should be delegated and not refer to ANC_MTH
	then these can be used for other things
###
class TripPlanOptionGroup


	TEMPLATE: require('Common/lib/listTemplates')


	###*
		Default method for tapping a checked rows
	###
	tapCheckedRow: (@index, @properties) ->
		GUI.deselectListViewItem(@listView, 0, @index)
		true


	###*
		Default method for tapping an unchecked row
	###
	tapUncheckedRow: (@index, @properties) ->
		GUI.deselectListViewItem(@listView, 0, @index)
		true

	###*
		@constructor
	###
	constructor: (title) ->
		self = this


		# Checked Item Template
		optionTemplateChecked = TEMPLATE.ONE_LINE_CHECK_MARK
		optionTemplateChecked.events = {

			"click": (e) => 
				index =  e.itemIndex
				properties = e.section.getItemAt(index).properties
				@tapCheckedRow(index, properties)
		}

		# Unchecked Item Template
		optionTemplate = TEMPLATE.ONE_LINE
		optionTemplate.events = {

			"click": (e) => 
				index = e.itemIndex
				properties = e.section.getItemAt(index).properties
				@tapUncheckedRow(index, properties)
		}

		# Container for group
		@view = 
		Ti.UI.createView({
				left: CNST.VIEW_LEFT_MRGN, 
				right:CNST.VIEW_RGHT_MRGN,
				bottom : CNST.VIEW_BTM_MRGN,
				height : Titanium.UI.SIZE,
				layout: 'vertical',
			});

		@titleBar = 
		new GUI.createItemHeader(title);

	  	# The List view
		@listView = 
		Ti.UI.createListView({
				left  : 0,
				top   : 0,
				height : THEME.TBL_ROW_HEIGHT,
				separatorColor : THEME.TBL_ROW_SPRTR_CLR,
				allowsSelection: true,
				backgroundColor: THEME.TBL_ROW_BGRND_CLR,
				showVerticalScrollIndicator: false,
				templates: { 
					"optionTemplateChecked": optionTemplateChecked,
					"optionTemplate": optionTemplate
				},
				defaultItemTemplate: "optionTemplate"            
			})

		@view.add(@titleBar)
		@view.add(@listView)



	###*
		Add items to the list group
	###
	addItems: (@itemList) ->

		listSection = Titanium.UI.createListSection({
    		items: itemList,
		})
		
		@listView.sections = [listSection]

		heightSupplement = 0

		if (Ti.Platform.osname is 'android')
			heightSupplement = 4

		@listView.setHeight (THEME.TBL_ROW_HEIGHT * itemList.length)+heightSupplement
		#do the android thing
		true


	###*
		Restore the user's previous selections
	###
	selectFromHistory: (@storedData, @IdLetter) ->
		
		if (!storedData)
			return	

		try
			newArray = storedData.filter (x) -> 

				if (x.idntfr?)
					return x.idntfr.charAt(0) is IdLetter

				return false

			for item,i in newArray
				@setRow(i,item.selected)

		catch e
			#we don't want it to crash if the data has become corrupt
			## clear the data so this doesn't happen again
			Ti.App.Properties.setList('TripPlanOptions', [])	

		true


	###*
		Set a particular row in the group to be checked or unchecked
	###			
	setRow: (@index,@value) ->

		item = @listView.sections[0].getItemAt(index)
		if (value)
			item.template = "optionTemplateChecked"
		else
			item.template = "optionTemplate"

		@listView.sections[0].updateItemAt(index, item)

		true


###*
	This class only allows a user to pick one item in the group and toggles
	the items on and off with respect to their selection
###
class ToggleSelectionGroup extends TripPlanOptionGroup

	
	tapUncheckedRow: (@index, @properties)->
		
		#Uncheck all other rows first
		for row, i in @listView.sections[0].items when row.template == "optionTemplateChecked"
			@setRow(i,off)
			ANC_MTHD.SetTripPlanOption(row.properties.identifier, false, row.properties.mask, row.properties.descr)

		@setRow(index,on)
		ANC_MTHD.SetTripPlanOption(properties.identifier, true, properties.mask, properties.descr)

		super
		true



###*
	This is the base class for selecting or deselecting multiple items
	within a trip plan option group
###
class RegularSelectionGroup extends TripPlanOptionGroup
	
	tapUncheckedRow: (@index, @properties)->
		@setRow(index,on)
		ANC_MTHD.SetTripPlanOption(properties.identifier, true, properties.mask, properties.descr)
		super
		true

	tapCheckedRow: (@index, @properties)->
		@setRow(index,off)
		ANC_MTHD.SetTripPlanOption(properties.identifier, false, properties.mask, properties.descr)
		super
		true


###*
	This class doesn't allow the user to deselect every single item
###
class AtleastOneSelectionGroup extends RegularSelectionGroup

	message = null

	setSelectOneMessage:(@_message) ->
		message = _message
		true

	tapCheckedRow: (@index, @properties)->
	
		selectedRowNumber = 0
		## todo - do this with a running total instead
		for item in @listView.sections[0].items when item.template is "optionTemplateChecked" or selectedRowNumber>1 
			selectedRowNumber +=1 

		if (selectedRowNumber>1)
			super
		else
			# The base class has been overridden so we need to deselect
			# the list view item manually in this case
			GUI.deselectListViewItem(@listView, 0, @index);

			if (message)
				alert(message)

		true

###*
	This is a special class for the accessibility requirments where sometimes
	it has a 'None' row
###
class RegularSelectionGroupOrNoneRowToggle extends RegularSelectionGroup
	

	###* if this row exists ... turn it on/off ###
	setRowWithMask: (@mask,@value)->
		for item,i in @listView.sections[0].items 
			if @mask == item.properties.mask
				@setRow(i,value)
				ANC_MTHD.SetTripPlanOption(item.identifier, value, @mask, item.properties.descr)



	tapCheckedRow: (@index, @properties)->

		if @properties.mask == "0"
			GUI.deselectListViewItem(@listView, 0, @index);
			return 

		selectedRowNumber = 0

		super

		for item in @listView.sections[0].items when item.template is "optionTemplateChecked" or selectedRowNumber>1 
			if item.properties.mask != "0" #ignore the 'none' row
				selectedRowNumber +=1 

		if selectedRowNumber < 1 #everything is now unchecked except the None row
			@setRowWithMask("0",on)
		else 
			super
		true

	tapUncheckedRow: (@index, @properties)->

		#if the user clicks the 'None' row it deselects the other doohickies
		if @properties.mask == "0"
			#Uncheck all other rows first
			for row, i in @listView.sections[0].items when row.template is "optionTemplateChecked"
				@setRow(i,off)
				ANC_MTHD.SetTripPlanOption(row.properties.identifier, false, row.properties.mask, row.properties.descr)

			@setRow(index,on)
			ANC_MTHD.SetTripPlanOption(properties.identifier, true, properties.mask, properties.descr)
			GUI.deselectListViewItem(@listView, 0, @index);
		else
			@setRowWithMask("0",off)
			super
		true



# Export the three classes to the global namespace for general consumption
root = exports ? this
root.RegularSelectionGroupOrNoneRowToggle = RegularSelectionGroupOrNoneRowToggle
root.RegularSelectionGroup = RegularSelectionGroup
root.ToggleSelectionGroup = ToggleSelectionGroup
root.AtleastOneSelectionGroup = AtleastOneSelectionGroup
